@@ -4,7 +4,7 @@ use warnings;
 package App::Spec;
 use 5.010;
 
-our $VERSION = '0.002'; # VERSION
+our $VERSION = '0.003'; # VERSION
 
 use App::Spec::Command;
 use App::Spec::Option;
@@ -29,6 +29,13 @@ has description => ( is => 'rw' );
 my $DATA = do { local $/; <DATA> };
 my $default_spec;
 
+sub build {
+    my ($class, $spec) = @_;
+    $_ = App::Spec::Option->build($_) for @{ $spec->{options} || [] };
+    $_ = App::Spec::Parameter->build($_) for @{ $spec->{parameters} || [] };
+    my $self = $class->new($spec);
+}
+
 sub _read_default_spec {
     $default_spec ||= YAML::XS::Load($DATA);
     return dclone $default_spec;
@@ -37,8 +44,10 @@ sub _read_default_spec {
 sub runner {
     my ($self) = @_;
     my $class = $self->class;
-    my $run = $class->new({
-        spec => $self
+    my $cmd = $class->new;
+    my $run = App::Spec::Run->new({
+        spec => $self,
+        cmd => $cmd,
     });
     return $run;
 }
@@ -49,25 +58,7 @@ sub read {
         die "No filename given";
     }
 
-    my $spec;
-    if (ref $file eq 'GLOB') {
-        my $data = do { local $/; <$file> };
-        $spec = eval { YAML::XS::Load($data) };
-    }
-    elsif (not ref $file) {
-        $spec = eval { YAML::XS::LoadFile($file) };
-    }
-    elsif (ref $file eq 'SCALAR') {
-        my $data = $$file;
-        $spec = eval { YAML::XS::Load($data) };
-    }
-    elsif (ref $file eq 'HASH') {
-        $spec = $file;
-    }
-
-    unless ($spec) {
-        die "Error reading '$file': $@";
-    }
+    my $spec = $class->load_data($file);
 
     my $has_subcommands = $spec->{subcommands} ? 1 : 0;
     my $default;
@@ -104,25 +95,35 @@ sub read {
         }
     }
 
-    my $self = $class->new({
-        name => $spec->{name},
-        appspec => $spec->{appspec},
-        class => $spec->{class},
-        title => $spec->{title},
-        markup => $spec->{markup},
-        options => [map {
-            App::Spec::Option->build($_)
-        } @{ $spec->{options} || [] }],
-        parameters => [map {
-            App::Spec::Parameter->build($_)
-        } @{ $spec->{parameters} || [] }],
-        subcommands => $commands,
-        abstract => $spec->{abstract},
-        description => $spec->{description},
-    });
+    $spec->{subcommands} = $commands;
+    my $self = $class->build($spec);
     return $self;
 }
 
+sub load_data {
+    my ($class, $file) = @_;
+    my $spec;
+    if (ref $file eq 'GLOB') {
+        my $data = do { local $/; <$file> };
+        $spec = eval { YAML::XS::Load($data) };
+    }
+    elsif (not ref $file) {
+        $spec = eval { YAML::XS::LoadFile($file) };
+    }
+    elsif (ref $file eq 'SCALAR') {
+        my $data = $$file;
+        $spec = eval { YAML::XS::Load($data) };
+    }
+    elsif (ref $file eq 'HASH') {
+        $spec = $file;
+    }
+
+    unless ($spec) {
+        die "Error reading '$file': $@";
+    }
+    return $spec;
+}
+
 sub _add_subcommands {
     my ($self, $commands1, $commands2, $ref) = @_;
     for my $name (keys %{ $commands2 || {} }) {
@@ -424,6 +425,8 @@ structure in perl.
 The L<App::Spec::Run> module is the framework which will run the actual
 app.
 
+Have a look at the L<App::Spec::Tutorial> for how to write an app.
+
 In the examples directory you will find the app C<myapp> which is supposed
 to demonstrate everything that App::Spec supports right now.
 
@@ -436,15 +439,18 @@ Your script:
     $run->run;
 
     # this is equivalent to
-    #my $run = Your::App->new({
+    #my $run = App::Spec::Run->new(
     #    spec => $spec,
-    #});
+    #    cmd => Your::App->new(
+    #        spec => $spec,
+    #    ),
+    #);
     #$run->run;
 
 Your App class:
 
     package Your::App;
-    use base 'App::Spec::Run';
+    use base 'App::Spec::Run::Cmd';
 
     sub command1 {
         my ($self) = @_;
@@ -462,6 +468,18 @@ Your App class:
 
     my $spec = App::Spec->read("/path/to/myapp-spec.yaml");
 
+=item load_data
+
+Takes a file, hashref or glob and returns generated appspec hashref
+
+    my $hash = $class->load_data($file);
+
+=item build
+
+Builds objects out of the hashref
+
+    my $appspec = App::Spec->build($hashref);
+
 =item runner
 
 Returns an instance of the your app class
@@ -511,6 +529,11 @@ Accessors for the things defined in the spec (file)
 
 L<App::AppSpec> - Utilities for App::Spec authors
 
+=head1 LICENSE
+
+This library is free software and may be distributed under the same terms
+as perl itself.
+
 =cut
 
 1;
